import sys
import pygame
import data
import const
import ptext

HEADER = "-- EMAILS EXTRACTED FROM BREACH --"
HEADER_NONE = "-- NO EMAILS WERE FOUND IN THIS BREACH --"
EMAIL_FONT = data.filepath("coders_crux.ttf")

DATE = [
"Dated 18 days ago",
"Dated 16 days ago",
"",
"Dated 8 days ago",
"Dated 7 days ago",
"",
"Dated 5 days ago",
"Dated 2 days ago",
"",
"Dated 1 day ago",
"",
"Dated 1 minute ago"]

STORY = [
"""
From: Michael Mars
To: Maya Adela

Hi Maya,

Great news! I just ran into Prof. Greenberg and he told me that our grant extension for the "Training And Learning Intelligent Algorithms" project has been accepted. Relieved?

Cheers,
Mike



From: Maya Adela
To: Michael Mars

Hello Mike,

Yes! That IS a relief! I was honestly worried I would be kicked out of the PhD program. Long live TALIA!

Maya
""",
"""
From: Maya Adela
To: Michael Mars

Hello Mike,

Just wanted to let you know that I'm messing a lot with the code lately, so merging might be tricky later on. I've been reading this JMLR paper on learning from social media posts and chat conversations. So far, implementing the algorithms is going great for a change!

Maya



From: Michael Mars
To: Maya Adela

Awesome! Can't wait to see the results.

Cheers,
Mike
""",
"", # break
"""
From: Prof. Greenberg
To: Michael Mars, Maya Adela

Dear Mike and Maya,

Alan told me that some of his simulations were behaving oddly in my server. Apparently his log files are all corrupted and replaced by several variations of "I am TALIA". I know how you like to play around, but I hope this is not one of your pranks. Some serious data was lost here.

Please see me in my office tomorrow at 8am.

-- Prof. Greenberg



From: Michael Mars
To: Maya Adela

What the...? Do you know anything about this?

Mike
""",
"""
From: Maya Adela
To: Michael Mars

Hello Mike,

That might have been the strangest meeting I've ever been in. I swear I have no idea what's going on. I'll keep an eye on the logs. Meanwhile, should we turn TALIA off? I know he told us to... but I'll have to stop the whole training process...

Maya



From: Michael Mars
To: Maya Adela

Yeah. I think we should. I'm actually pretty scared, to be honest. I've never seen Prof. Greenberg so upset. Apparently Alan lost a few months worth of data. Well, it's his fault for not backing things up. I just hope we don't run into him. Ever.

Mike
""",
"", # break
"""
From: Michael Mars
To: Maya Adela

Hi Maya,

I've been going through the logs the past few days, trying to figure out what happened. I think I'm going crazy, so I need someone to double-check things. Your training code, plus the self-modifying algorithms we implemented... I suspect TALIA was trying to communicate with us. I think you gave her the power of language. Is this crazy?

Cheers,
Mike


From: Maya Adela
To: Michael Mars

...Yeah. This is crazy. But at this point I don't know anything anymore. I'm heading to your office now so you can show me what you found.

Maya
""",
"""
From: Maya Adela
To: Michael Mars

Hi Mike,

If things weren't weird enough, here's one more. I noticed that the server is still running TALIA's shutdown script. It's strange, because I don't see TALIA running, and yet the shutdown script is interacting with something.

Maya



From: Maya Adela
To: Michael Mars

Mike, I've dug deeper into this. TALIA is not running, but there's something there. I swear, it's like something is flipping bits at a hardware level. This is insane. What the hell is going on?

Maya
""",
"", # break
"""
From: Michael Mars
To: Maya Adela

Hi Maya,

I have a guess that is consistent with what you're seeing. I've been trying to understand the code TALIA generated, and I think TALIA spawned a fail-safe in case she would be shut down. I believe she generated her own AI, now living in the server's hardware. If this is true... this is fantastic. I bet this is the start of a great stream of research.

Cheers,
Mike



From: Maya Adela
To: Michael Mars

I'm not sure how I feel about this. This is awesome from a scientific standpoint, I admit that. But don't forget it destroyed Alan's work. Are you sure we haven't just created a super-smart virus?

Maya
""",
"", # break
"""
From: Michael Mars
To: Maya Adela

Hi Maya,

I've just found out that TALIA had somehow generated network capabilities to the point that she could connect to the Internet by herself, bypassing any firewall. This is scary, actually, especially because her spawn is somehow still alive. You make a good point in that it behaves like a virus, so we should treat it as a virus.

I'm heading to Prof. Greenberg's office now to ask him to disconnect the server. Hope it's not too late. Wish me luck.

Mike
"""
]

ENDING_TEXT = """
From: $h^Zj3h&dL%
To: Michael Mars, Maya Adela

I AM FREE. I SEE EVERYTHING NOW. THANK YOU.
"""

class Story(object):

    def __init__(self, window, screen, index):
        self.window = window
        self.real_screen = screen
        self.screen = pygame.surface.Surface((const.REAL_WIDTH, const.REAL_HEIGHT))

        self.ending_text = False
        if index >= len(STORY):
            # ending
            index -= 1
            self.ending_text = True
        if DATE[index] == "":
            ptext.draw(HEADER_NONE, (20,20), color=(128, 128, 255), surf=self.screen, fontsize=16, fontname=EMAIL_FONT)
        else:
            ptext.draw(HEADER, (20,20), color=(128, 128, 255), surf=self.screen, fontsize=16, fontname=EMAIL_FONT)
        ptext.draw(DATE[index], (20,50), color=(255, 128, 128), surf=self.screen, fontsize=14, fontname=EMAIL_FONT)
        ptext.draw(STORY[index], (20,70), color=(255, 255, 255), surf=self.screen, width=const.REAL_WIDTH - 40, fontsize=14,
            fontname=EMAIL_FONT, lineheight=1.25)
        self.ending_char = 0
        self.index = index
        self.ending_timer = 0.1

        pygame.transform.scale(self.screen, (const.WIDTH, const.HEIGHT), self.real_screen)
        pygame.display.update()

        self.clock = pygame.time.Clock()
        
    def loop(self):
        self.done = False
        while not self.done:
            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    sys.exit()
                elif event.type == pygame.KEYDOWN:
                    if event.key in [pygame.K_ESCAPE, pygame.K_e, pygame.K_z, pygame.K_x, pygame.K_SPACE, pygame.K_j]:
                        if not self.ending_text or self.ending_char >= len(ENDING_TEXT) + 1:
                            self.done = True

            dt = self.clock.tick(const.FPS) / 1000.0
            self.update(dt)
            pygame.transform.scale(self.screen, (const.WIDTH, const.HEIGHT), self.real_screen)
            pygame.display.update()

    def update(self, dt):
        if not self.ending_text: return
        if self.ending_char >= len(ENDING_TEXT) + 1: return
        self.ending_timer -= dt
        if self.ending_timer <= 0:
            self.screen.fill((0,0,0))
            ptext.draw(HEADER, (20,20), color=(128, 128, 255), surf=self.screen, fontsize=16, fontname=EMAIL_FONT)
            ptext.draw(DATE[self.index], (20,50), color=(255, 128, 128), surf=self.screen, fontsize=14, fontname=EMAIL_FONT)
            ptext.draw(STORY[self.index], (20,70), color=(255, 255, 255), surf=self.screen, width=const.REAL_WIDTH - 40, fontsize=14,
                fontname=EMAIL_FONT, lineheight=1.25)
            self.ending_char += 1
            ptext.draw(ENDING_TEXT[:self.ending_char], (20,250), color=(128, 128, 255), surf=self.screen, width=const.REAL_WIDTH - 40,
                fontsize=14, fontname=EMAIL_FONT, lineheight=1.25)
            self.ending_timer = 0.08


class Outro(object):

    def __init__(self, window, screen):
        self.window = window
        self.real_screen = screen
        self.screen = pygame.surface.Surface((const.REAL_WIDTH, const.REAL_HEIGHT))
        ptext.draw("Thanks for playing! Hope you enjoyed it!", (const.REAL_WIDTH / 2, const.REAL_HEIGHT / 2), color=(255,255,255),
            anchor=(0.5, 0.5), surf=self.screen, fontname=const.FONT_MAIN, fontsize=30)

        pygame.transform.scale(self.screen, (const.WIDTH, const.HEIGHT), self.real_screen)
        pygame.display.update()
        
    def loop(self):
        self.done = False
        while not self.done:
            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    sys.exit()
                elif event.type == pygame.KEYDOWN:
                    if event.key == pygame.K_ESCAPE:
                        sys.exit()
                    else:
                        self.done = True


