import pygame
import const


class Animation(object):
    
    def __init__(self, sprite, frames, duration, initial_frame_num=0, stopped=False):
        self.sprite = sprite
        self.frames = frames
        self.current_frame = initial_frame_num - 1
        self.duration = duration
        self.timer = 0
        self.stopped = stopped

    def run(self, delay=0):
        self.timer += delay
        self.stopped = False

    def pause(self):
        self.stopped = True

    def set_start_time(self, start_time):
        self.current_frame += int((start_time + 1e-6) / self.duration)
        self.current_frame = self.current_frame % len(self.frames)
        self.timer = (1000 * start_time) % (1000 * self.duration) / 1000.0
    
    def update(self, dt):
        if self.stopped: return
        self.timer -= dt
        if self.timer <= 0:
            self.timer = self.duration
            self.current_frame += 1
            if self.current_frame >= len(self.frames):
                self.current_frame = 0
            self.sprite.image = self.frames[self.current_frame]


DIRS = ['t', 'b', 'l', 'r']

def get_tile_at_xy(x, y):
    return (int(x / const.TILE_SIZE), int(y / const.TILE_SIZE))

def get_xy_at_tile(x, y):
    return (x * const.TILE_SIZE, y * const.TILE_SIZE)

def get_tile_from_dir(x, y, dir):
    if dir == 'b':
        if y >= const.NTILES_Y - 1: return None
        return (x, y+1)
    elif dir == 't':
        if y <= 0: return None
        return (x, y-1)
    elif dir == 'r':
        if x >= const.NTILES_X - 1: return None
        return (x+1, y)
    elif dir == 'l':
        if x <= 0: return None
        return (x-1, y)
    return None

def opposite_dir(d):
    return {'t': 'b', 'b': 't', 'l': 'r', 'r': 'l',
            'tr':'bl', 'bl':'tr', 'tl':'br', 'br':'tl'}[d]